@@ -1,3 +1,4 @@
+import glob
 import time
 import json
 import os
@@ -59,6 +60,7 @@ class ZestRunnerBase:
         exclude_groups=None,
         common_tmp=None,
         tmp_root="/tmp",
+        preview=False,
         **kwargs,
     ):
         """
@@ -131,10 +133,19 @@ class ZestRunnerBase:
         self.root = root or os.getcwd()
         assert self.root[0] == os.sep
 
+        allow_list = self.allow_to_run.split(":")
+
+        for r in allow_list:
+            if r == "__failed__":
+                # load_previous is a bit slow so we only want to do it if requested
+                prev_fails = self.load_previous()
+                allow_list += prev_fails
+                break
+
         self.root_zests, self.allow_to_run, find_errors = zest_finder.find_zests(
             root,
             include_dirs,
-            self.allow_to_run.split(":"),
+            allow_list,
             allow_files.split(":") if allow_files is not None else None,
             match_string,
             exclude_string,
@@ -143,6 +154,13 @@ class ZestRunnerBase:
             exclude_groups.split(":") if exclude_groups is not None else None,
         )
 
+        if preview:
+            for i in sorted(self.allow_to_run):
+                print(i)
+            print(f"\nWould have run {len(self.allow_to_run)} tests")
+            self.retcode = 1
+            return
+
         self.handle_find_errors(find_errors)
 
     def handle_find_errors(self, find_errors):
@@ -161,3 +179,15 @@ class ZestRunnerBase:
             and self.match_string is None
             and self.groups is None
         )
+
+    def load_previous(self):
+        fails = {}
+        for file in glob.glob(str(self.output_folder / "*")):
+            with open(file) as f:
+                for line in f:
+                    res = json.loads(line)
+                    # There can be multiple records from previous runs,
+                    # accept the LASt state of the error run
+                    fails[res.get("full_name")] = True if res.get("error") is not None else False
+        return list(set([key for key, val in fails.items() if val]))
+
